@@ -3,7 +3,7 @@
 /***********************************************************************
 *  This code is part of GLPK (GNU Linear Programming Kit).
 *
-*  Copyright (C) 2000, 01, 02, 03, 04, 05, 06, 07 Andrew Makhorin,
+*  Copyright (C) 2000, 01, 02, 03, 04, 05, 06, 07, 08 Andrew Makhorin,
 *  Department for Applied Informatics, Moscow Aviation Institute,
 *  Moscow, Russia. All rights reserved. E-mail: <mao@mai2.rcnet.ru>.
 *
@@ -22,6 +22,7 @@
 ***********************************************************************/
 
 #include "glpios.h"
+#define xfault xerror
 
 /* CAUTION: DO NOT CHANGE THE LIMITS BELOW */
 
@@ -53,15 +54,10 @@
 *  The routine returns a pointer to the object created, which should be
 *  used in any subsequent operations on this object. */
 
-glp_prob *glp_create_prob(void)
-{     glp_prob *lp;
-      lp = xmalloc(sizeof(glp_prob));
-      lp->pool = dmp_create_pool();
+static void create_prob(glp_prob *lp)
+{     lp->pool = dmp_create_pool();
       lp->cps = xmalloc(sizeof(struct LPXCPS));
       lpx_reset_parms(lp);
-#if 0
-      lp->str_buf = xmalloc(255+1);
-#endif
       lp->tree = NULL;
       /* LP/MIP data */
       lp->name = NULL;
@@ -91,6 +87,13 @@ glp_prob *glp_create_prob(void)
       /* integer solution (MIP) */
       lp->mip_stat = GLP_UNDEF;
       lp->mip_obj = 0.0;
+      return;
+}
+
+glp_prob *glp_create_prob(void)
+{     glp_prob *lp;
+      lp = xmalloc(sizeof(glp_prob));
+      create_prob(lp);
       return lp;
 }
 
@@ -243,7 +246,11 @@ int glp_add_rows(glp_prob *lp, int nrs)
          row->ptr = NULL;
          row->rii = 1.0;
          row->stat = GLP_BS;
+#if 0
          row->bind = -1;
+#else
+         row->bind = 0;
+#endif
          row->prim = row->dual = 0.0;
          row->pval = row->dval = 0.0;
          row->mipx = 0.0;
@@ -318,7 +325,11 @@ int glp_add_cols(glp_prob *lp, int ncs)
          col->ptr = NULL;
          col->sjj = 1.0;
          col->stat = GLP_NS;
+#if 0
          col->bind = -1;
+#else
+         col->bind = 0; /* the basis may remain valid */
+#endif
          col->prim = col->dual = 0.0;
          col->pval = col->dval = 0.0;
          col->mipx = 0.0;
@@ -1120,6 +1131,31 @@ void glp_del_cols(glp_prob *lp, int ncs, const int num[])
       return;
 }
 
+/***********************************************************************
+*  NAME
+*
+*  glp_erase_prob - erase problem object content
+*
+*  SYNOPSIS
+*
+*  void glp_erase_prob(glp_prob *lp);
+*
+*  DESCRIPTION
+*
+*  The routine glp_erase_prob erases the content of the specified
+*  problem object. The effect of this operation is the same as if the
+*  problem object would be deleted with the routine glp_delete_prob and
+*  then created anew with the routine glp_create_prob, with exception
+*  that the handle (pointer) to the problem object remains valid. */
+
+static void delete_prob(glp_prob *lp);
+
+void glp_erase_prob(glp_prob *lp)
+{     delete_prob(lp);
+      create_prob(lp);
+      return;
+}
+
 /***********************************************************************
 *  NAME
 *
@@ -1134,12 +1170,9 @@ void glp_del_cols(glp_prob *lp, int ncs, const int num[])
 *  The routine glp_delete_prob deletes the specified problem object and
 *  frees all the memory allocated to it. */
 
-void glp_delete_prob(glp_prob *lp)
+static void delete_prob(glp_prob *lp)
 {     dmp_delete_pool(lp->pool);
       xfree(lp->cps);
-#if 0
-      xfree(lp->str_buf);
-#endif
       xassert(lp->tree == NULL);
       xfree(lp->row);
       xfree(lp->col);
@@ -1148,6 +1181,11 @@ void glp_delete_prob(glp_prob *lp)
       xfree(lp->bhead);
       if (lp->bfcp != NULL) xfree(lp->bfcp);
       if (lp->bfd != NULL) bfd_delete_it(lp->bfd);
+      return;
+}
+
+void glp_delete_prob(glp_prob *lp)
+{     delete_prob(lp);
       xfree(lp);
       return;
 }
